@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2014  Internet Systems Consortium, Inc. ("ISC")
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -17,9 +17,7 @@
 SYSTEMTESTTOP=../..
 . $SYSTEMTESTTOP/conf.sh
 
-SYSTESTDIR=legacy
-
-echo_i "sign edns512"
+echo "I:sign edns512"
 
 zone=edns512
 infile=edns512.db.in
