@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2010, 2012, 2014  Internet Systems Consortium, Inc. ("ISC")
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -15,5 +15,6 @@
 # PERFORMANCE OF THIS SOFTWARE.
 
 rm -f K* ns1/K* keyset-* dsset-* ns1/*.db ns1/*.signed ns1/*.jnl
-rm -f dig.out
-rm -f ns1/key ns1/named.memstats
+rm -f dig.out pin
+rm -f ns1/*.key ns1/named.memstats
+rm -f supported
