@@ -1,6 +1,7 @@
 #!/usr/bin/perl -w
 #
-# Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2004-2008, 2010-2015  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2001  Internet Software Consortium.
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -14,6 +15,8 @@
 # OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 # PERFORMANCE OF THIS SOFTWARE.
 
+# $Id: start.pl,v 1.30 2012/02/06 23:46:44 tbox Exp $
+
 # Framework for starting test servers.
 # Based on the type of server specified, check for port availability, remove
 # temporary files, start the server, and verify that the server is running.
@@ -24,60 +27,38 @@ use Cwd;
 use Cwd 'abs_path';
 use Getopt::Long;
 
-# Usage:
-#   perl start.pl [--noclean] [--restart] [--port port] test [server [options]]
+# Option handling
+#   --noclean test [server [options]]
 #
-#   --noclean       Do not cleanup files in server directory.
-#
-#   --restart       Indicate that the server is being restarted, so get the
-#                   server to append output to an existing log file instead of
-#                   starting a new one.
-#
-#   --port port     Specify the default port being used by the server to answer
-#                   queries (default 5300).  This script will interrogate the
-#                   server on this port to see if it is running. (Note: for
-#                   "named" nameservers, this can be overridden by the presence
-#                   of the file "named.port" in the server directory containing
-#                   the number of the query port.)
-#
-#   test            Name of the test directory.
-#
-#   server          Name of the server directory.  This will be of the form
-#                   "nsN" or "ansN", where "N" is an integer between 1 and 8.
-#                   If not given, the script will start all the servers in the
-#                   test directory.
-#
-#   options         Alternate options for the server.
-#
-#                   NOTE: options must be specified with '-- "<option list>"',
-#                   for instance: start.pl . ns1 -- "-c n.conf -d 43"
-#
-#                   ALSO NOTE: this variable will be filled with the contents
-#                   of the first non-commented/non-blank line of args in a file
-#                   called "named.args" in an ns*/ subdirectory. Only the FIRST
-#                   non-commented/non-blank line is used (everything else in
-#                   the file is ignored). If "options" is already set, then
-#                   "named.args" is ignored.
+#   --noclean - Do not cleanup files in server directory
+#   test - name of the test directory
+#   server - name of the server directory
+#   options - alternate options for the server
+#             NOTE: options must be specified with '-- "<option list>"',
+#              for instance: start.pl . ns1 -- "-c n.conf -d 43"
+#             ALSO NOTE: this variable will be filled with the
+#		contents of the first non-commented/non-blank line of args
+#		in a file called "named.args" in an ns*/ subdirectory only
+#		the FIRST non-commented/non-blank line is used (everything
+#		else in the file is ignored. If "options" is already set,
+#		then "named.args" is ignored.
 
-my $usage = "usage: $0 [--noclean] [--restart] [--port <port>] test-directory [server-directory [server-options]]";
+my $usage = "usage: $0 [--noclean] [--restart] test-directory [server-directory [server-options]]";
 my $noclean = '';
 my $restart = '';
-my $queryport = 5300;
-
-GetOptions('noclean' => \$noclean, 'restart' => \$restart, 'port=i' => \$queryport) or die "$usage\n";
-
+GetOptions('noclean' => \$noclean, 'restart' => \$restart);
 my $test = $ARGV[0];
 my $server = $ARGV[1];
 my $options = $ARGV[2];
 
 if (!$test) {
-	die "$usage\n";
+	print "$usage\n";
 }
 if (!-d $test) {
-	die "No test directory: \"$test\"\n";
+	print "No test directory: \"$test\"\n";
 }
 if ($server && !-d "$test/$server") {
-	die "No server directory: \"$test/$server\"\n";
+	print "No server directory: \"$test/$server\"\n";
 }
 
 # Global variables
@@ -115,6 +96,7 @@ if ($server) {
 	foreach $name(@ns, @lwresd, @ans) {
 		&start_server($name);
 		&verify_server($name) if ($name =~ /^ns/);
+		
 	}
 }
 
@@ -123,7 +105,7 @@ if ($server) {
 sub check_ports {
 	my $server = shift;
 	my $options = "";
-	my $port = $queryport;
+	my $port = 5300;
 	my $file = "";
 
 	$file = $testdir . "/" . $server . "/named.port" if ($server);
@@ -149,7 +131,7 @@ sub check_ports {
 		if (++$tries > 4) {
 			print "$0: could not bind to server addresses, still running?\n";
 			print "I:server sockets not available\n";
-			print "I:failed\n";
+			print "R:FAIL\n";
 			system("$PERL $topdir/stop.pl $testdir"); # Is this the correct behavior?
 			exit 1;
 		}
@@ -198,19 +180,20 @@ sub start_server {
 			close FH;
 			$command .= "$options";
 		} else {
+			$command .= "-D $server ";
 			$command .= "-m record,size,mctx ";
 			$command .= "-T clienttest ";
-			$command .= "-T nosoa "
+			$command .= "-T nosoa " 
 				if (-e "$testdir/$server/named.nosoa");
-			$command .= "-T noaa "
+			$command .= "-T noaa " 
 				if (-e "$testdir/$server/named.noaa");
-			$command .= "-T noedns "
+			$command .= "-T noedns " 
 				if (-e "$testdir/$server/named.noedns");
-			$command .= "-T dropedns "
+			$command .= "-T dropedns " 
 				if (-e "$testdir/$server/named.dropedns");
-			$command .= "-T maxudp512 "
+			$command .= "-T maxudp512 " 
 				if (-e "$testdir/$server/named.maxudp512");
-			$command .= "-T maxudp1460 "
+			$command .= "-T maxudp1460 " 
 				if (-e "$testdir/$server/named.maxudp1460");
 			$command .= "-c named.conf -d 99 -g -U 4";
 		}
@@ -252,7 +235,7 @@ sub start_server {
 	} elsif ($server =~ /^ans/) {
 		$cleanup_files = "{ans.run}";
                 if (-e "$testdir/$server/ans.py") {
-                        $command = "$PYTHON -u ans.py 10.53.0.$' $queryport";
+                        $command = "$PYTHON ans.py 10.53.0.$' 5300";
                 } elsif (-e "$testdir/$server/ans.pl") {
                         $command = "$PERL ans.pl";
                 } else {
@@ -271,7 +254,7 @@ sub start_server {
 		$pid_file = "ans.pid";
 	} else {
 		print "I:Unknown server type $server\n";
-		print "I:failed\n";
+		print "R:FAIL\n";
 		system "$PERL $topdir/stop.pl $testdir";
 		exit 1;
 	}
@@ -298,7 +281,7 @@ sub start_server {
 	while (!-s $pid_file) {
 		if (++$tries > 140) {
 			print "I:Couldn't start server $server (pid=$child)\n";
-			print "I:failed\n";
+			print "R:FAIL\n";
 			system "kill -9 $child" if ("$child" ne "");
 			system "$PERL $topdir/stop.pl $testdir";
 			exit 1;
@@ -314,7 +297,7 @@ sub start_server {
 sub verify_server {
 	my $server = shift;
 	my $n = $server;
-	my $port = $queryport;
+	my $port = 5300;
 	my $tcp = "+tcp";
 
 	$n =~ s/^ns//;
@@ -336,8 +319,9 @@ sub verify_server {
 		my $return = system("$DIG $tcp +noadd +nosea +nostat +noquest +nocomm +nocmd +noedns -p $port version.bind. chaos txt \@10.53.0.$n > dig.out");
 		last if ($return == 0);
 		if (++$tries >= 30) {
+			print `grep ";" dig.out > /dev/null`;
 			print "I:no response from $server\n";
-			print "I:failed\n";
+			print "R:FAIL\n";
 			system("$PERL $topdir/stop.pl $testdir");
 			exit 1;
 		}
