@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2005, 2007, 2009, 2014  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004, 2005, 2007, 2009, 2014, 2015  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 2000, 2001  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -52,7 +52,7 @@
 #ifdef ISC_PLATFORM_OPENSSLHASH
 void
 isc_md5_init(isc_md5_t *ctx) {
-	EVP_DigestInit(ctx, EVP_md5());
+	RUNTIME_CHECK(EVP_DigestInit(ctx, EVP_md5()) == 1);
 }
 
 void
@@ -62,12 +62,16 @@ isc_md5_invalidate(isc_md5_t *ctx) {
 
 void
 isc_md5_update(isc_md5_t *ctx, const unsigned char *buf, unsigned int len) {
-	EVP_DigestUpdate(ctx, (const void *) buf, (size_t) len);
+	if (len == 0U)
+		return;
+	RUNTIME_CHECK(EVP_DigestUpdate(ctx,
+				       (const void *) buf,
+				       (size_t) len) == 1);
 }
 
 void
 isc_md5_final(isc_md5_t *ctx, unsigned char *digest) {
-	EVP_DigestFinal(ctx, digest, NULL);
+	RUNTIME_CHECK(EVP_DigestFinal(ctx, digest, NULL) == 1);
 }
 
 #elif PKCS11CRYPTO
